#!/usr/bin/env python
__all__ = """
test_dynalign       test_raxml          test_stride  
test_blast          test_foldalign      test_rnaalifold
test_carnac         test_ilm            test_rnaforester
test_clustalw       test_knetfold       test_rnaview
test_cmfinder       test_mfold          test_sfold
test_comrna         test_muscle         test_unafold
test_consan         test_nupack         test_util
test_contrafold     test_parameters     test_vienna_package
test_cove           test_pfold          test_gctmpca
test_dialign        test_pknotsrg       test_fasttree
test_msms""".split()

__author__ = ""
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Micah Hamady", "Rob Knight", "Catherine Lozupone",
                    "Sandra Smit", "Gavin Huttley", "Greg Caporaso",
                    "Marcin Cieslik"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Rob Knight"
__email__ = "rob@spot.colorado.edu"
__status__ = "Production"

