#!/usr/bin/env python
"""Seqsim: code for simulating sequences and trees."""

__all__ = ['analysis',
           'birth_death',
           'markov',
           'microarray',
           'microarray_normalize',
           'randomization',
           'searchpath',
           'sequence_generators',
           'tree',
           'usage']

__author__ = "Rob Knight"
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Rob Knight", "Daniel McDonald", "Mike Robeson", \
               "Jesse Zaneveld"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Rob Knight"
__email__ = "rob@spot.colorado.edu"
__status__ = "Production"
