#!/usr/bin/env python
__all__ = ['test_goodness_of_fit', 'test_UPGMA', 'test_metric_scaling', 'test_procrustes', 'test_nmds', 'test_approximate_mds']

__author__ = ""
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Catherine Lozuopone", "Peter Maxwell", "Rob Knight", "Justin Kuczynski"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Rob Knight"
__email__ = "rob@spot.colorado.edu"
__status__ = "Production"
