#!/usr/bin/env python

__all__ = ['alignment', 'alphabet', 'annotation', 'bitvector', 'entity',
           'genetic_code', 'info', 'location', 'moltype', 'profile', 'sequence',
           'tree', 'usage']

__author__ = ""
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Jeremy Widmann", "Gavin Huttley", "Rob Knight",
                    "Sandra Smit", "Peter Maxwell", "Matthew Wakefield",
                    "Greg Caporaso", "Marcin Cieslik" ]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Gavin Huttley"
__email__ = "gavin.huttley@anu.edu.au"
__status__ = "Production"
