#!/usr/bin/env python

__all__ = ['fit_function'
           'function_optimisation',
           'geometry',
           'markov',
           'matrix_exponential_integration',
           'matrix_exponentiation',
           'matrix_logarithm',
           'optimiser',
           'optimisers',
           'scipy_optimisers',
           'scipy_optimize',
           'simannealingoptimiser',
           'solve',
           'spatial',
           'svd']


__author__ = ""
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Gavin Huttley", "Peter Maxwell", "Matthew Wakefield",
                    "Rob Knight", "Edward Lang", "Sandra Smit",
                    "Antonio Gonzalez Pena", "Ben Kaehler"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Gavin Huttley"
__email__ = "gavin.huttley@anu.edu.au"
__status__ = "Production"
