#!/usr/bin/env python
__all__ = ['test_distribution_plots']

__author__ = ""
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Jeremy Widmann", "Sandra Smit", "Gavin Huttley",
                    "Rob Knight", "Zongzhi Liu", "Amanda Birmingham",
                    "Greg Caporaso", "Jai Ram Rideout"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Rob Knight"
__email__ = "rob@spot.colorado.edu"
__status__ = "Production"
