#!/usr/bin/env python
__all__ = ['test_rna2d', 'test_annotation', 'test_selection', 'test_asa',
           'test_manipulation', 'test_contact']

__author__ = ""
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Rob Knight", "Sandra Smit", "Marcin Cieslik"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Rob Knight"
__email__ = "rob@spot.colorado.edu"
__status__ = "Production"
