#!/usr/bin/env python
__all__ = ['test_mage', 'test_fasta', 'test_pdb_color', 'test_xyzrn']

__author__ = ""
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Rob Knight", "Gavin Huttley", "Sandra Smit",
               "Marcin Cieslik", "Jeremy Widmann"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Gavin Huttley"
__email__ = "gavin.huttley@anu.edu.au"
__status__ = "Production"
