#!/usr/bin/env python
__all__ = ['test_aaindex',
           'test_agilent_microarray',
           'test_blast',
           'test_bpseq',
           'test_cigar',
           'test_clustal',
           'test_cutg',
           'test_dialign',
           'test_ebi',
           'test_fasta',
           'test_fastq',
           'test_genbank',
           'test_illumina_sequence',
           'test_locuslink',
           'test_mage',
           'test_meme',
           'test_msms',
           'test_ncbi_taxonomy',
           'test_nexus',
           'test_pdb',
           'test_structure',
           'test_phylip',
           'test_record',
           'test_record_finder',
           'test_rfam',
           'test_rna_fold',
           'test_rnaview',
           'test_sprinzl',
           'test_tree',
           'test_unigene'
           'test_stride']

__author__ = ""
__copyright__ = "Copyright 2007-2016, The Cogent Project"
__credits__ = ["Jeremy Widmann", "Catherine Lozuopone", "Gavin Huttley",
                    "Rob Knight", "Sandra Smit", "Micah Hamady",
                    "Jeremy Widmann", "Hua Ying", "Greg Caporaso",
                    "Zongzhi Liu", "Jason Carnes", "Peter Maxwell",
                    "Marcin Cieslik"]
__license__ = "GPL"
__version__ = "1.9"
__maintainer__ = "Rob Knight"
__email__ = "rob@spot.colorado.edu"
__status__ = "Production"

